#!/usr/bin/perl

use strict;
use Sendmail::Milter;
use Getopt::Long;
use File::Basename;
use Sys::Syslog qw(:DEFAULT setlogsock);
use IO::Socket::UNIX;

my($version) = '0.01';
my($author) = 'Victor Ustugov aka corvax';
my($email) = 'victor@corvax.kiev.ua';
my($info) = "backup perl mail filter $version 2003 by $author $email";

my($default_syslog_facility) = 'local4';
my($MDA) = '';

my(@sendmail_config_defaults) = (
    '/etc/sendmail.cf',
    '/etc/mail/sendmail.cf'
);

sub header_callback {
    my $ctx = shift;
    my $headerf = shift;
    my $headerv = shift;

    my($message_ref) = $ctx->getpriv();
    ${$message_ref} .= "$headerf: $headerv\n";
    $ctx->setpriv($message_ref);

    return SMFIS_CONTINUE;
}

sub eoh_callback {
    my $ctx = shift;

    my($message_ref) = $ctx->getpriv();
    ${$message_ref} .= "\n";
    $ctx->setpriv($message_ref);

    return SMFIS_CONTINUE;
}

sub body_callback {
    my $ctx = shift;
    my $body_part = shift;

    my($message_ref) = $ctx->getpriv();
    ${$message_ref} .= $body_part;
    $ctx->setpriv($message_ref);

    return SMFIS_CONTINUE;
}

sub eom_callback {
    my $ctx = shift;
    my $message_ref = $ctx->getpriv();

    unless (open(MDA, "|$MDA")) {
	syslog('notice', "couldn't execute MDA\n");
    } else {
	print(MDA ${$message_ref});
	syslog('notice', "couldn't send copy of message to the MDA\n") unless (close(MDA));
    }

    $ctx->setpriv(undef);

    return SMFIS_CONTINUE;
}

sub abort_callback {
    my $ctx = shift;
    $ctx->setpriv(undef);
    return SMFIS_CONTINUE;
}

sub close_callback {
    my $ctx = shift;
    $ctx->setpriv(undef);
    return SMFIS_CONTINUE;
}

my %my_callbacks = (
    'header'	=> \&header_callback,
    'eoh'	=> \&eoh_callback,
    'body'	=> \&body_callback,
    'eom'	=> \&eom_callback,
    'abort'	=> \&abort_callback,
    'close'	=> \&close_callback,
);

BEGIN: {
    my($milter_name);
    my($sendmail_config);
    my($syslog_facility);
    my($help);
    my($version);

    #    
    GetOptions (
        'help'			=> \$help,
        'version'		=> \$version,
        'milter-name=s'		=> \$milter_name,
        'sendmail-config=s'	=> \$sendmail_config,
        'syslog-facility=s'	=> \$syslog_facility,
        'MDA=s'			=> \$MDA
    );

    $syslog_facility = $default_syslog_facility unless ($syslog_facility);

    my($script_name) = basename($0);

    if ($help) {
        print(
"$info

Usage: perl $script_name [OPTIONS]

General options:
  --milter-name        name of mail filter in sendmail.cf
  --sendmail-config    path to the sendmail.cf
  --MDA                command line for MDA

Additional options:
  --syslog-facility    syslog facility

Miscellaneous:
  --version            display the version of $script_name and exit
  --help               display this help and exit
");
        exit();
    }
    if ($version) {
        print("$info\n");
        exit();
    }

    #    sendmail',        
    $sendmail_config = find_sendmail_config(@sendmail_config_defaults) unless ($sendmail_config);

    #     ,       
    unless ($milter_name and $sendmail_config and $MDA) {
	print("name of mail filter in sendmail.cf is missing") unless ($milter_name);
        print("path to the sendmail.cf is missing") unless ($sendmail_config);
        print("command line for MDA is missing") unless ($MDA);
        print("

Usage: perl $script_name [OPTIONS]

Try `$script_name --help' for more options.

");
        exit();
    }

    setlogsock('unix');
    openlog($milter_name, 'cons,pid', $syslog_facility);

    my($conn) = Sendmail::Milter::auto_getconn($milter_name, $sendmail_config);

    print_msg("Found connection info for '$milter_name': $conn");

    if ($conn =~ /^local:(.+)$/) {
	my $unix_socket = $1;
	if (-e $unix_socket) {
	    print_msg("Attempting to unlink UNIX socket '$conn'...");
	    if (unlink($unix_socket) == 0) {
		print_msg("failed");
		exit();
	    }
	    print_msg("successful");
	}
    }

    if (not Sendmail::Milter::auto_setconn($milter_name, $sendmail_config)) {
	print_msg("Failed to detect connection information");
    } elsif (not Sendmail::Milter::register($milter_name, \%my_callbacks, SMFI_CURR_ACTS)) {
	print_msg("Failed to register callbacks for '$milter_name'");
    } else {
	print_msg("Starting Sendmail::Milter $Sendmail::Milter::VERSION engine");
	if (Sendmail::Milter::main()) {
	    print_msg("Successful exit from the Sendmail::Milter engine");
	} else {
	    print_msg("Unsuccessful exit from the Sendmail::Milter engine");
	}
    }
    closelog();
}

sub print_msg() {
    my($msg) = @_;
    print($msg."\n");
    syslog('notice', $msg);
}

sub find_sendmail_config {
    my(@sendmail_config_defaults) = @_;
    my($file);
    foreach $file (@sendmail_config_defaults) {
        if (-f $file) {
            return($file);
        }
    }
    return('');
}
